import { Test, TestingModule } from '@nestjs/testing';
import { getLoggerToken, PinoLogger } from 'nestjs-pino';
import { join } from 'path';
import { DeployByPm2Service } from './deploy-by-pm2.service';

describe('DeployByPm2Service', () => {
  let service: DeployByPm2Service;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        DeployByPm2Service,
        {
          provide: getLoggerToken(DeployByPm2Service.name),
          useValue: new PinoLogger({
            pinoHttp: {
              level: 'silent',
            },
          }),
        },
      ],
    }).compile();

    service = module.get<DeployByPm2Service>(DeployByPm2Service);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('getAppsSn', () => {
    it('should return right value', () => {
      expect(
        service['getAppsSn']([
          { name: 'app' },
          { name: 'app#4' },
          { name: 'app#1' },
        ]),
      ).toEqual(4 + 1);
    });
    it('should return 1 when no match', () => {
      expect(
        service['getAppsSn']([
          { name: 'bar' },
          { name: 'foo#4' },
          { name: 'foo#1' },
        ]),
      ).toEqual(4 + 1);
    });
  });

  describe('filterOldApps', () => {
    it('should return right value', () => {
      expect(
        service['filterOldApps'](
          [{ name: 'app' }],
          [
            { name: 'app' },
            { name: 'app#4' },
            { name: 'foo#2' },
            { name: 'bar' },
          ],
        ),
      ).toEqual([{ name: 'app' }, { name: 'app#4' }]);
    });
    it('should return [] when no match', () => {
      expect(
        service['filterOldApps'](
          [{ name: 'app' }],
          [{ name: 'foo#2' }, { name: 'bar' }],
        ),
      ).toEqual([]);
    });
  });

  describe('replaceAppName', () => {
    it('should be replaced with right value', () => {
      const getAppsSn = jest
        .spyOn(service, 'getAppsSn' as any)
        .mockImplementation(() => 1);
      const options = [{ name: 'app' }, { name: 'foo' }];
      service['replaceAppName'](options, []);
      expect(options).toEqual([{ name: 'app#1' }, { name: 'foo#1' }]);
      expect(getAppsSn).toBeCalledTimes(1);
      expect(getAppsSn.mock.calls[0][0]).toEqual([]);
    });
  });

  describe('deploy', () => {
    it('should be success', async () => {
      const filterOldApps = jest
        .spyOn(service, 'filterOldApps' as any)
        .mockImplementation(() => [{ name: 'app#1' }, { name: 'app#2' }]);
      const replaceAppName = jest
        .spyOn(service, 'replaceAppName' as any)
        .mockImplementation((options) => (options[0].name = 'app#2'));
      const stopApps = jest
        .spyOn(service, 'stopApps' as any)
        .mockImplementation(() => Promise.resolve());
      await expect(
        service['deploy'](
          join(
            __dirname,
            '../../../../test/__mocks__/deploy-service/ecosystem.config.js',
          ),
          join(__dirname, '../../../../test/__mocks__/deploy-service'),
        ),
      ).resolves.toBeFalsy();
      expect(filterOldApps).toBeCalledTimes(1);
      expect(replaceAppName).toBeCalledTimes(1);
      expect(stopApps).toBeCalledTimes(1);
      stopApps.mockReset();

      await service['stopApps']([{ name: 'app#2' }]);
    }, 10_000);
  });
});
